/*= driver mtx3240 ========================================================*/
/* Original Release: 05/11/2001                                            */
/* By: O.L.                                                                */
/* Originally written in C                                                 */
/*=========================================================================*/

#include "rs232.h"
#include "formatio.h"
#include "utility.h"
#include "mtx3240.h"

#define TEMPO	0.1

/*= STATIC VARIABLES ======================================================*/
static int portCom;


/*= UTILITY ROUTINES ======================================================*/
int sendData (char *commande);
int readData (char *buff);


/*=========================================================================*/
/* Cette fonction ouvre le port srie , interroge l'appareil connect au   */
/* PC, reset l'appareil.                                                   */
/*=========================================================================*/
int mtx3240_init (int serialPort, int IDQuery, int resetDevice)
{
	int mtx3240_err = 0;
	char buff[50];

    OpenComConfig (serialPort, "", 19200, 0, 8, 1, 512, 512);
    
    if (rs232err == 0)
    	{
    	portCom = serialPort;
	    SetComTime (portCom, 1.0);
	    SetXMode (portCom, 0);
		SetCTSMode (portCom, LWRS_HWHANDSHAKE_OFF);
		
		if (IDQuery)
			{
			sendData ("*IDN?\r");
			readData (buff);
			if (FindPattern (buff, 0, -1, "METRIX, MTX3240, ", 0, 0) == -1)
				mtx3240_err = -1;
			}

		if (resetDevice)
			{
			sendData ("*RST\r");
			}
        }
    else
		mtx3240_err = rs232err;
		
	return mtx3240_err;
}


/*=========================================================================*/
/* Cette fonction configure le type de signal, l'amplitude, l'offset,      */
/* le rapport cyclique, les niveaux logiques et le frequencemetre          */
/*=========================================================================*/
int mtx3240_config (int function, double frequency, double amplitude, 
			double offset, int duty, double logicHigh, double logicLow)
{
	int mtx3240_err = 0;
	static char *Function[6] = {"SIN","TRI","SQU","LOGI","DC","FREQ"};
	char message[200];

	Fmt (message, "FUNC %s\r", Function[function]);
	sendData (message);

	if (function < 5)
		{
		if (function == 4)
			{
			Fmt (message, "VOLT:OFFS %f\r", offset);
			sendData (message);
			}
		else
			{
			Fmt (message, "FREQ %f\r", frequency);
			sendData (message);
			Fmt (message, "PULS:DCYC %i\r", duty);
			sendData (message);

			if (function != 3)
				{
				Fmt (message, "VOLT %f\r", amplitude);
				sendData (message);
				Fmt (message, "VOLT:OFFS %f\r", offset);
				sendData (message);
				}
			else
				{
				Fmt (message, "VOLT:HIGH %f\r", logicHigh);
				sendData (message);
				Fmt (message, "VOLT:LOW %f\r", logicLow);
				sendData (message);
				}
			}
		}
	
	return mtx3240_err;
}

/*=========================================================================*/
/* Cette fonction configure le Sweep, interne ou externe, lineaire ou      */
/* logarythmique, la dure, la frequence max                               */
/*=========================================================================*/
int mtx3240_sweep (int enable, double startFrequency,double stopFrequency,
					double time, int mode, int source)
{
	int mtx3240_err = 0;
	static char *Sweep[2] = {"FIX","SWE"};
	static char *Source[2] = {"INT","EXT"};
	static char *Mode[2] = {"LIN","LOG"};
	char message[200];
	
	Fmt (message, "FREQ:MODE %s\r", Sweep[enable]);
	sendData (message);
	
	if (enable)
		{
		Fmt (message, "FREQ:STAR %f\r", startFrequency);
		sendData (message);
		Fmt (message, "SWE:SOUR %s\r", Source[source]);
		sendData (message);
		
		if (source == 0)
			{
			Fmt (message, "FREQ:STOP %f\r", stopFrequency);
			sendData (message);
			Fmt (message, "SWE:TIME:LLIM %f\r", time);
			sendData (message);
			Fmt (message, "SWE:SPAC %s\r", Mode[mode]);
			sendData (message);
			}
		}

	return mtx3240_err;
}

/*=========================================================================*/
/* Cette fonction permet de lire la valeur du frequencemetre               */
/*																		   */
/*=========================================================================*/
int mtx3240_measure (double *frequency)
{
	int mtx3240_err = 0;
	char buff[100];

	sendData ("MEAS?\r");
	readData (buff);
	Scan (buff, "%f", frequency);

	return mtx3240_err;
}

/*=========================================================================*/
/* Cette function ferme le port srie                                      */
/*                                                                         */
/*=========================================================================*/
int mtx3240_close (void)
{
	int mtx3240_err = 0;

    CloseCom (portCom);
    portCom = 0;
    
	return mtx3240_err;
}


/*=========================================================================*/
/* cette fonction crit dans un buffer de data la chaine de commande       */
/*  envoyer                                                               */
/*=========================================================================*/
int sendData (char *commande)
{
	int mtx3240_err = 0;

	FlushInQ (portCom);
	FlushOutQ (portCom);
    ComWrt (portCom, commande, StringLength (commande));
    Delay (TEMPO);
    
    if (rs232err != 0)
		mtx3240_err = rs232err;
		
	return mtx3240_err;
}


/*=========================================================================*/
/* cette fonction lit dans un buffer la chaine de commande renvoye        */
/*                                                                         */
/*=========================================================================*/
int readData (char *buff)
{
	int mtx3240_err = 0;

    ComRdTerm (portCom, buff, 50, 0x0D);

    if (rs232err != 0)
		mtx3240_err = rs232err;
		
	return mtx3240_err;
}

